@@ -9,7 +9,7 @@
 #' @details
 #' List vctrs are special cases. When created through `new_vctr()`, the
 #' resulting list vctr should always be recognized as a list by
-#' `vec_is_list()`. Because of this, if `inherit_base_type` is `FALSE`
+#' `obj_is_list()`. Because of this, if `inherit_base_type` is `FALSE`
 #' an error is thrown.
 #'
 #' @section Base methods:
@@ -64,10 +64,12 @@
 #' @export
 #' @keywords internal
 #' @aliases vctr
-new_vctr <- function(.data,
-                     ...,
-                     class = character(),
-                     inherit_base_type = NULL) {
+new_vctr <- function(
+  .data,
+  ...,
+  class = character(),
+  inherit_base_type = NULL
+) {
   if (!is_vector(.data)) {
     abort("`.data` must be a vector type.")
   }
@@ -103,12 +105,11 @@ names_repair_missing <- function(x) {
     return(x)
   }
 
-  missing <- vec_equal_na(x)
-
-  if (any(missing)) {
+  if (vec_any_missing(x)) {
     # We never want to allow `NA_character_` names to slip through, but
     # erroring on them has caused issues. Instead, we repair them to the
     # empty string (#784).
+    missing <- vec_detect_missing(x)
     x <- vec_assign(x, missing, "")
   }
 
@@ -138,12 +139,14 @@ vec_cast.vctrs_vctr <- function(x, to, ...) {
   UseMethod("vec_cast.vctrs_vctr")
 }
 
-vctr_cast <- function(x,
-                      to,
-                      ...,
-                      x_arg = "",
-                      to_arg = "",
-                      call = caller_env()) {
+vctr_cast <- function(
+  x,
+  to,
+  ...,
+  x_arg = "",
+  to_arg = "",
+  call = caller_env()
+) {
   # These are not strictly necessary, but make bootstrapping a new class
   # a bit simpler
   if (is.object(x)) {
@@ -234,8 +237,9 @@ diff.vctrs_vctr <- function(x, lag = 1L, differences = 1L, ...) {
   stopifnot(length(differences) == 1L, differences >= 1L)
 
   n <- vec_size(x)
-  if (lag * differences >= n)
+  if (lag * differences >= n) {
     return(vec_slice(x, 0L))
+  }
 
   out <- x
   for (i in seq_len(differences)) {
@@ -312,7 +316,7 @@ as.character.vctrs_vctr <- function(x, ...) {
 as.list.vctrs_vctr <- function(x, ...) {
   out <- vec_chop(x)
 
-  if (vec_is_list(x)) {
+  if (obj_is_list(x)) {
     out <- lapply(out, `[[`, 1)
   }
 
@@ -356,11 +360,13 @@ as.data.frame2 <- function(x) {
 }
 
 #' @export
-as.data.frame.vctrs_vctr <- function(x,
-                                     row.names = NULL,
-                                     optional = FALSE,
-                                     ...,
-                                     nm = paste(deparse(substitute(x), width.cutoff = 500L), collapse = " ")) {
+as.data.frame.vctrs_vctr <- function(
+  x,
+  row.names = NULL,
+  optional = FALSE,
+  ...,
+  nm = paste(deparse(substitute(x), width.cutoff = 500L), collapse = " ")
+) {
   force(nm)
 
   if (has_dim(x)) {
@@ -404,15 +410,13 @@ as.difftime.vctrs_vctr <- function(x, units = "secs", ...) {
 
 #' @export
 is.na.vctrs_vctr <- function(x) {
-  vec_equal_na(x)
+  vec_detect_missing(x)
 }
 
 #' @importFrom stats na.fail
 #' @export
 na.fail.vctrs_vctr <- function(object, ...) {
-  missing <- vec_equal_na(object)
-
-  if (any(missing)) {
+  if (vec_any_missing(object)) {
     # Return the same error as `na.fail.default()`
     abort("missing values in object")
   }
@@ -436,13 +440,12 @@ na_remove <- function(x, type) {
   # The only difference between `na.omit()` and `na.exclude()` is the class
   # of the `na.action` attribute
 
-  missing <- vec_equal_na(x)
-
-  if (!any(missing)) {
+  if (!vec_any_missing(x)) {
     return(x)
   }
 
   # `na.omit/exclude()` attach the locations of the omitted values to the result
+  missing <- vec_detect_missing(x)
   loc <- which(missing)
 
   names <- vec_names(x)
@@ -460,12 +463,10 @@ na_remove <- function(x, type) {
 }
 
 #' @export
-anyNA.vctrs_vctr <- if (getRversion() >= "3.2") {
-  function(x, recursive = FALSE) {
-    any(is.na(x))
-  }
-} else {
-  function(x) {
+anyNA.vctrs_vctr <- function(x, recursive = FALSE) {
+  if (recursive && obj_is_list(x)) {
+    any(map_lgl(x, anyNA, recursive = recursive))
+  } else {
     any(is.na(x))
   }
 }
@@ -756,21 +757,25 @@ new_hidden <- function(x = double()) {
   stopifnot(is.numeric(x))
   new_vctr(vec_cast(x, double()), class = "hidden", inherit_base_type = FALSE)
 }
+
+#' @export
 format.hidden <- function(x, ...) rep("xxx", length(x))
 
 local_hidden <- function(frame = caller_env()) {
-  local_bindings(.env = global_env(), .frame = frame,
-    vec_ptype2.hidden.hidden  = function(x, y, ...) new_hidden(),
-    vec_ptype2.hidden.double  = function(x, y, ...) new_hidden(),
-    vec_ptype2.double.hidden  = function(x, y, ...) new_hidden(),
+  local_bindings(
+    .env = global_env(),
+    .frame = frame,
+    vec_ptype2.hidden.hidden = function(x, y, ...) new_hidden(),
+    vec_ptype2.hidden.double = function(x, y, ...) new_hidden(),
+    vec_ptype2.double.hidden = function(x, y, ...) new_hidden(),
     vec_ptype2.hidden.logical = function(x, y, ...) new_hidden(),
     vec_ptype2.logical.hidden = function(x, y, ...) new_hidden(),
 
-    vec_cast.hidden.hidden   = function(x, to, ...) x,
-    vec_cast.hidden.double   = function(x, to, ...) new_hidden(vec_data(x)),
-    vec_cast.double.hidden   = function(x, to, ...) vec_data(x),
-    vec_cast.hidden.logical  = function(x, to, ...) new_hidden(as.double(x)),
-    vec_cast.logical.hidden  = function(x, to, ...) as.logical(vec_data(x))
+    vec_cast.hidden.hidden = function(x, to, ...) x,
+    vec_cast.hidden.double = function(x, to, ...) new_hidden(vec_data(x)),
+    vec_cast.double.hidden = function(x, to, ...) vec_data(x),
+    vec_cast.hidden.logical = function(x, to, ...) new_hidden(as.double(x)),
+    vec_cast.logical.hidden = function(x, to, ...) as.logical(vec_data(x))
   )
 }
 
