@@ -15,8 +15,8 @@ extern SEXP vctrs_field_get(SEXP, SEXP);
 extern SEXP vctrs_field_set(SEXP, SEXP, SEXP);
 extern SEXP vctrs_fields(SEXP);
 extern SEXP vctrs_n_fields(SEXP);
-extern SEXP vctrs_hash(SEXP);
-extern SEXP vctrs_hash_object(SEXP);
+extern r_obj* ffi_vec_hash(r_obj*);
+extern r_obj* ffi_obj_hash(r_obj*);
 extern SEXP vctrs_equal_object(SEXP, SEXP);
 extern SEXP vctrs_duplicated(SEXP);
 extern SEXP vctrs_unique_loc(SEXP);
@@ -27,7 +27,7 @@ extern SEXP vec_split(SEXP, SEXP);
 extern SEXP vctrs_group_id(SEXP);
 extern SEXP vctrs_group_rle(SEXP);
 extern SEXP vec_group_loc(SEXP);
-extern SEXP vctrs_equal(SEXP, SEXP, SEXP);
+extern SEXP ffi_vec_equal(SEXP, SEXP, SEXP, SEXP, SEXP);
 extern r_obj* ffi_vec_detect_missing(r_obj*);
 extern r_obj* ffi_vec_any_missing(r_obj* x);
 extern r_obj* ffi_vec_compare(r_obj*, r_obj*, r_obj*);
@@ -43,6 +43,7 @@ extern SEXP vctrs_typeof(SEXP, SEXP);
 extern r_obj* ffi_obj_is_vector(r_obj*);
 extern r_obj* ffi_obj_check_vector(r_obj*, r_obj*);
 extern r_obj* ffi_vec_check_size(r_obj*, r_obj*, r_obj*);
+extern r_obj* ffi_vec_check_recyclable(r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_ptype2(r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_typeof2(r_obj*, r_obj*);
 extern r_obj* ffi_typeof2_s3(r_obj*, r_obj*);
@@ -64,14 +65,14 @@ extern SEXP vec_proxy_order(SEXP);
 extern r_obj* ffi_df_proxy(r_obj*, r_obj*);
 extern SEXP vctrs_unspecified(SEXP);
 extern r_obj* ffi_ptype(r_obj*, r_obj*, r_obj*);
-extern SEXP vec_ptype_finalise(SEXP);
+extern r_obj* vec_ptype_finalise(r_obj*);
 extern r_obj* ffi_minimal_names(r_obj*);
 extern r_obj* ffi_unique_names(r_obj*, r_obj*);
 extern SEXP ffi_as_minimal_names(SEXP);
 extern SEXP vec_names(SEXP);
 extern SEXP vctrs_is_unique_names(SEXP);
 extern SEXP vctrs_as_unique_names(SEXP, SEXP);
-extern SEXP vec_set_names(SEXP, SEXP);
+extern SEXP ffi_vec_set_names(SEXP, SEXP);
 extern r_obj* ffi_df_cast_opts(r_obj*, r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_df_ptype2_opts(r_obj*, r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_type_info(r_obj*);
@@ -79,19 +80,18 @@ extern SEXP ffi_proxy_info(SEXP);
 extern r_obj* ffi_class_type(r_obj*);
 extern r_obj* ffi_vec_bare_df_restore(r_obj*, r_obj*);
 extern r_obj* ffi_recycle(r_obj*, r_obj*, r_obj*);
-extern r_obj* ffi_assign(r_obj*, r_obj*, r_obj*, r_obj*);
-extern r_obj* ffi_assign_seq(r_obj*, r_obj*, r_obj*, r_obj*, r_obj*);
-extern SEXP vctrs_set_attributes(SEXP, SEXP);
+extern r_obj* ffi_assign(r_obj*, r_obj*, r_obj*, r_obj*, r_obj*);
+extern r_obj* ffi_assign_seq(r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*);
+extern r_obj* ffi_assign_compact_condition(r_obj*, r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_as_df_row(r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_outer_names(r_obj*, r_obj*, r_obj*);
 extern SEXP vctrs_df_size(SEXP);
 extern r_obj* ffi_as_df_col(r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_apply_name_spec(r_obj*, r_obj*, r_obj*, r_obj*);
-extern r_obj* ffi_unset_s4(r_obj*);
+extern r_obj* ffi_as_not_s4(r_obj*);
 extern SEXP vctrs_validate_name_repair_arg(SEXP);
 extern SEXP vctrs_validate_minimal_names(SEXP, SEXP);
 extern r_obj* ffi_vec_as_names(r_obj*, r_obj*, r_obj*, r_obj*);
-extern r_obj* ffi_is_partial(r_obj*);
 extern r_obj* ffi_obj_is_list(r_obj*);
 extern SEXP vctrs_try_catch_callback(SEXP, SEXP);
 extern r_obj* ffi_is_coercible(r_obj*, r_obj*, r_obj*, r_obj*);
@@ -102,7 +102,7 @@ extern r_obj* df_flatten(r_obj*);
 extern SEXP vctrs_linked_version(void);
 extern r_obj* ffi_tib_ptype2(r_obj*, r_obj*, r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_tib_cast(r_obj*, r_obj*, r_obj*, r_obj*, r_obj*);
-extern r_obj* ffi_assign_params(r_obj*, r_obj*, r_obj*, r_obj*);
+extern r_obj* ffi_assign_params(r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*);
 extern SEXP vctrs_has_dim(SEXP);
 extern r_obj* ffi_vec_rep(r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_vec_rep_each(r_obj*, r_obj*, r_obj*);
@@ -130,9 +130,10 @@ extern SEXP vctrs_slice_complete(SEXP);
 extern SEXP vctrs_locate_complete(SEXP);
 extern SEXP vctrs_detect_complete(SEXP);
 extern SEXP vctrs_normalize_encoding(SEXP);
+extern r_obj* ffi_chr_is_normalized(r_obj*);
 extern SEXP vctrs_order(SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP vctrs_locate_sorted_groups(SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP vctrs_order_info(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+extern SEXP vctrs_order_info(SEXP, SEXP, SEXP, SEXP, SEXP);
 extern r_obj* ffi_vec_unrep(r_obj*, r_obj*);
 extern SEXP vctrs_fill_missing(SEXP, SEXP, SEXP);
 extern r_obj* ffi_chr_paste_prefix(r_obj*, r_obj*, r_obj*);
@@ -141,7 +142,7 @@ extern r_obj* vctrs_integer64_proxy(r_obj*);
 extern r_obj* vctrs_integer64_restore(r_obj*);
 extern r_obj* vctrs_list_drop_empty(r_obj*);
 extern r_obj* vctrs_is_altrep(r_obj* x);
-extern r_obj* ffi_interleave_indices(r_obj*, r_obj*);
+extern r_obj* ffi_list_interleave(r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_compute_nesting_container_info(r_obj*, r_obj*);
 extern r_obj* ffi_locate_matches(r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_interval_groups(r_obj*, r_obj*, r_obj*, r_obj*);
@@ -149,24 +150,43 @@ extern r_obj* ffi_interval_locate_groups(r_obj*, r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_interval_complement(r_obj*, r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_interval_locate_containers(r_obj*, r_obj*);
 extern r_obj* ffi_check_list(r_obj*, r_obj*);
-extern r_obj* ffi_list_all_vectors(r_obj*, r_obj*);
-extern r_obj* ffi_list_check_all_vectors(r_obj*, r_obj*);
+extern r_obj* ffi_list_all_vectors(r_obj*, r_obj*, r_obj*);
+extern r_obj* ffi_list_check_all_vectors(r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_as_short_length(r_obj*, r_obj*);
 extern r_obj* ffi_s3_get_method(r_obj*, r_obj*, r_obj*);
-extern r_obj* ffi_list_all_size(r_obj*, r_obj*, r_obj*);
-extern r_obj* ffi_list_check_all_size(r_obj*, r_obj*, r_obj*);
+extern r_obj* ffi_list_all_size(r_obj*, r_obj*, r_obj*, r_obj*);
+extern r_obj* ffi_list_check_all_size(r_obj*, r_obj*, r_obj*, r_obj*);
+extern r_obj* ffi_list_all_recyclable(r_obj*, r_obj*, r_obj*, r_obj*);
+extern r_obj* ffi_list_check_all_recyclable(r_obj*, r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_vec_set_intersect(r_obj*, r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_vec_set_difference(r_obj*, r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_vec_set_union(r_obj*, r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_vec_set_symmetric_difference(r_obj*, r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_vec_expand_grid(r_obj*, r_obj*, r_obj*, r_obj*);
+extern r_obj* ffi_list_combine(r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*);
+extern r_obj* ffi_compact_seq(r_obj*, r_obj*, r_obj*);
+extern r_obj* ffi_as_compact_condition(r_obj*);
+extern r_obj* ffi_vec_case_when(r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*);
+extern r_obj* ffi_vec_replace_when(r_obj*, r_obj*, r_obj*, r_obj*);
+extern r_obj* ffi_vec_recode_values(r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*);
+extern r_obj* ffi_vec_replace_values(r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*);
+extern r_obj* ffi_vec_if_else(r_obj*, r_obj*, r_obj*, r_obj*, r_obj*, r_obj*);
+extern r_obj* ffi_vec_pany(r_obj*, r_obj*, r_obj*, r_obj*);
+extern r_obj* ffi_vec_pall(r_obj*, r_obj*, r_obj*, r_obj*);
 
 
 // Maturing
 // In the public header
-extern bool obj_is_vector(SEXP);
-extern R_len_t short_vec_size(SEXP);
-extern SEXP short_vec_recycle(SEXP, R_len_t);
+extern bool maturing_obj_is_vector(SEXP);
+extern R_len_t maturing_short_vec_size(SEXP);
+extern SEXP maturing_short_vec_recycle(SEXP, R_len_t);
+
+// Defunct
+// Previously were in the public header, but have since been removed
+// Must be removed from the public header but still exist as a C callable for 1 CRAN release of vctrs,
+// so that dependent CRAN binaries (slider or tibblify) can be rebuilt with the new public header that
+// no longer looks for these callables.
+extern bool defunct_vec_is_vector(SEXP);
 
 // Experimental
 // Exported but not available in the public header
@@ -176,7 +196,7 @@ extern SEXP exp_vec_slice_impl(SEXP, SEXP);
 extern SEXP exp_vec_names(SEXP);
 extern SEXP exp_vec_set_names(SEXP, SEXP);
 extern SEXP exp_short_compact_seq(R_len_t, R_len_t, bool);
-extern SEXP exp_short_init_compact_seq(int*, R_len_t, R_len_t, bool);
+extern void exp_short_init_compact_seq(int*, R_len_t, R_len_t, bool);
 
 // Defined below
 SEXP vctrs_init_library(SEXP);
@@ -189,7 +209,7 @@ void vctrs_init_altrep_rle(DllInfo*);
 // Defined in altrep-lazy-character.c
 extern r_obj* ffi_altrep_new_lazy_character(r_obj*);
 extern r_obj* ffi_altrep_lazy_character_is_materialized(r_obj*);
-extern r_obj* vctrs_init_altrep_lazy_character(DllInfo*);
+extern void vctrs_init_altrep_lazy_character(DllInfo*);
 
 static const R_CallMethodDef CallEntries[] = {
   {"vctrs_list_get",                            (DL_FUNC) &vctrs_list_get, 2},
@@ -198,8 +218,8 @@ static const R_CallMethodDef CallEntries[] = {
   {"vctrs_field_set",                           (DL_FUNC) &vctrs_field_set, 3},
   {"vctrs_fields",                              (DL_FUNC) &vctrs_fields, 1},
   {"vctrs_n_fields",                            (DL_FUNC) &vctrs_n_fields, 1},
-  {"vctrs_hash",                                (DL_FUNC) &vctrs_hash, 1},
-  {"vctrs_hash_object",                         (DL_FUNC) &vctrs_hash_object, 1},
+  {"ffi_vec_hash",                              (DL_FUNC) &ffi_vec_hash, 1},
+  {"ffi_obj_hash",                              (DL_FUNC) &ffi_obj_hash, 1},
   {"vctrs_equal_object",                        (DL_FUNC) &vctrs_equal_object, 2},
   {"vctrs_unique_loc",                          (DL_FUNC) &vctrs_unique_loc, 1},
   {"vctrs_duplicated",                          (DL_FUNC) &vctrs_duplicated, 1},
@@ -216,7 +236,7 @@ static const R_CallMethodDef CallEntries[] = {
   {"vctrs_dim",                                 (DL_FUNC) &vctrs_dim, 1},
   {"vctrs_dim_n",                               (DL_FUNC) &vctrs_dim_n, 1},
   {"vctrs_is_unspecified",                      (DL_FUNC) &vctrs_is_unspecified, 1},
-  {"vctrs_equal",                               (DL_FUNC) &vctrs_equal, 3},
+  {"ffi_vec_equal",                             (DL_FUNC) &ffi_vec_equal, 5},
   {"ffi_vec_detect_missing",                    (DL_FUNC) &ffi_vec_detect_missing, 1},
   {"ffi_vec_any_missing",                       (DL_FUNC) &ffi_vec_any_missing, 1},
   {"ffi_vec_compare",                           (DL_FUNC) &ffi_vec_compare, 3},
@@ -227,6 +247,7 @@ static const R_CallMethodDef CallEntries[] = {
   {"ffi_obj_is_vector",                         (DL_FUNC) &ffi_obj_is_vector, 1},
   {"ffi_obj_check_vector",                      (DL_FUNC) &ffi_obj_check_vector, 2},
   {"ffi_vec_check_size",                        (DL_FUNC) &ffi_vec_check_size, 3},
+  {"ffi_vec_check_recyclable",                  (DL_FUNC) &ffi_vec_check_recyclable, 3},
   {"ffi_ptype2",                                (DL_FUNC) &ffi_ptype2, 3},
   {"ffi_typeof2",                               (DL_FUNC) &ffi_typeof2, 2},
   {"ffi_typeof2_s3",                            (DL_FUNC) &ffi_typeof2_s3, 2},
@@ -257,7 +278,7 @@ static const R_CallMethodDef CallEntries[] = {
   {"vctrs_names",                               (DL_FUNC) &vec_names, 1},
   {"vctrs_is_unique_names",                     (DL_FUNC) &vctrs_is_unique_names, 1},
   {"vctrs_as_unique_names",                     (DL_FUNC) &vctrs_as_unique_names, 2},
-  {"vctrs_set_names",                           (DL_FUNC) &vec_set_names, 2},
+  {"ffi_vec_set_names",                         (DL_FUNC) &ffi_vec_set_names, 2},
   {"ffi_df_cast_opts",                          (DL_FUNC) &ffi_df_cast_opts, 4},
   {"ffi_df_ptype2_opts",                        (DL_FUNC) &ffi_df_ptype2_opts, 4},
   {"ffi_type_info",                             (DL_FUNC) &ffi_type_info, 1},
@@ -265,15 +286,15 @@ static const R_CallMethodDef CallEntries[] = {
   {"ffi_class_type",                            (DL_FUNC) &ffi_class_type, 1},
   {"ffi_vec_bare_df_restore",                   (DL_FUNC) &ffi_vec_bare_df_restore, 2},
   {"ffi_recycle",                               (DL_FUNC) &ffi_recycle, 3},
-  {"ffi_assign",                                (DL_FUNC) &ffi_assign, 4},
-  {"ffi_assign_seq",                            (DL_FUNC) &ffi_assign_seq, 5},
-  {"vctrs_set_attributes",                      (DL_FUNC) &vctrs_set_attributes, 2},
+  {"ffi_assign",                                (DL_FUNC) &ffi_assign, 5},
+  {"ffi_assign_seq",                            (DL_FUNC) &ffi_assign_seq, 6},
+  {"ffi_assign_compact_condition",              (DL_FUNC) &ffi_assign_compact_condition, 4},
   {"ffi_as_df_row",                             (DL_FUNC) &ffi_as_df_row, 3},
   {"ffi_outer_names",                           (DL_FUNC) &ffi_outer_names, 3},
   {"vctrs_df_size",                             (DL_FUNC) &vctrs_df_size, 1},
   {"ffi_as_df_col",                             (DL_FUNC) &ffi_as_df_col, 3},
   {"ffi_apply_name_spec",                       (DL_FUNC) &ffi_apply_name_spec, 4},
-  {"ffi_unset_s4",                              (DL_FUNC) &ffi_unset_s4, 1},
+  {"ffi_as_not_s4",                             (DL_FUNC) &ffi_as_not_s4, 1},
   {"vctrs_altrep_rle_Make",                     (DL_FUNC) &altrep_rle_Make, 1},
   {"vctrs_altrep_rle_is_materialized",          (DL_FUNC) &altrep_rle_is_materialized, 1},
   {"ffi_altrep_new_lazy_character",             (DL_FUNC) &ffi_altrep_new_lazy_character, 1},
@@ -281,7 +302,6 @@ static const R_CallMethodDef CallEntries[] = {
   {"vctrs_validate_name_repair_arg",            (DL_FUNC) &vctrs_validate_name_repair_arg, 1},
   {"vctrs_validate_minimal_names",              (DL_FUNC) &vctrs_validate_minimal_names, 2},
   {"ffi_vec_as_names",                          (DL_FUNC) &ffi_vec_as_names, 4},
-  {"ffi_is_partial",                            (DL_FUNC) &ffi_is_partial, 1},
   {"ffi_obj_is_list",                           (DL_FUNC) &ffi_obj_is_list, 1},
   {"vctrs_try_catch_callback",                  (DL_FUNC) &vctrs_try_catch_callback, 2},
   {"ffi_is_coercible",                          (DL_FUNC) &ffi_is_coercible, 4},
@@ -292,7 +312,7 @@ static const R_CallMethodDef CallEntries[] = {
   {"vctrs_linked_version",                      (DL_FUNC) &vctrs_linked_version, 0},
   {"ffi_tib_ptype2",                            (DL_FUNC) &ffi_tib_ptype2, 5},
   {"ffi_tib_cast",                              (DL_FUNC) &ffi_tib_cast, 5},
-  {"ffi_assign_params",                         (DL_FUNC) &ffi_assign_params, 4},
+  {"ffi_assign_params",                         (DL_FUNC) &ffi_assign_params, 6},
   {"vctrs_has_dim",                             (DL_FUNC) &vctrs_has_dim, 1},
   {"ffi_vec_rep",                               (DL_FUNC) &ffi_vec_rep, 3},
   {"ffi_vec_rep_each",                          (DL_FUNC) &ffi_vec_rep_each, 3},
@@ -320,9 +340,10 @@ static const R_CallMethodDef CallEntries[] = {
   {"vctrs_locate_complete",                     (DL_FUNC) &vctrs_locate_complete, 1},
   {"vctrs_detect_complete",                     (DL_FUNC) &vctrs_detect_complete, 1},
   {"vctrs_normalize_encoding",                  (DL_FUNC) &vctrs_normalize_encoding, 1},
+  {"ffi_chr_is_normalized",                     (DL_FUNC) &ffi_chr_is_normalized, 1},
   {"vctrs_order",                               (DL_FUNC) &vctrs_order, 5},
   {"vctrs_locate_sorted_groups",                (DL_FUNC) &vctrs_locate_sorted_groups, 5},
-  {"vctrs_order_info",                          (DL_FUNC) &vctrs_order_info, 6},
+  {"vctrs_order_info",                          (DL_FUNC) &vctrs_order_info, 5},
   {"ffi_vec_unrep",                             (DL_FUNC) &ffi_vec_unrep, 2},
   {"vctrs_fill_missing",                        (DL_FUNC) &vctrs_fill_missing, 3},
   {"ffi_chr_paste_prefix",                      (DL_FUNC) &ffi_chr_paste_prefix, 3},
@@ -331,7 +352,7 @@ static const R_CallMethodDef CallEntries[] = {
   {"vctrs_integer64_restore",                   (DL_FUNC) &vctrs_integer64_restore, 1},
   {"vctrs_list_drop_empty",                     (DL_FUNC) &vctrs_list_drop_empty, 1},
   {"vctrs_is_altrep",                           (DL_FUNC) &vctrs_is_altrep, 1},
-  {"ffi_interleave_indices",                    (DL_FUNC) &ffi_interleave_indices, 2},
+  {"ffi_list_interleave",                       (DL_FUNC) &ffi_list_interleave, 6},
   {"ffi_compute_nesting_container_info",        (DL_FUNC) &ffi_compute_nesting_container_info, 2},
   {"ffi_locate_matches",                        (DL_FUNC) &ffi_locate_matches, 14},
   {"ffi_interval_groups",                       (DL_FUNC) &ffi_interval_groups, 4},
@@ -339,23 +360,35 @@ static const R_CallMethodDef CallEntries[] = {
   {"ffi_interval_complement",                   (DL_FUNC) &ffi_interval_complement, 4},
   {"ffi_interval_locate_containers",            (DL_FUNC) &ffi_interval_locate_containers, 2},
   {"ffi_check_list",                            (DL_FUNC) &ffi_check_list, 2},
-  {"ffi_list_all_vectors",                      (DL_FUNC) &ffi_list_all_vectors, 2},
-  {"ffi_list_check_all_vectors",                (DL_FUNC) &ffi_list_check_all_vectors, 2},
+  {"ffi_list_all_vectors",                      (DL_FUNC) &ffi_list_all_vectors, 3},
+  {"ffi_list_check_all_vectors",                (DL_FUNC) &ffi_list_check_all_vectors, 3},
   {"ffi_as_short_length",                       (DL_FUNC) &ffi_as_short_length, 2},
   {"ffi_s3_get_method",                         (DL_FUNC) &ffi_s3_get_method, 3},
-  {"ffi_list_all_size",                         (DL_FUNC) &ffi_list_all_size, 3},
-  {"ffi_list_check_all_size",                   (DL_FUNC) &ffi_list_check_all_size, 3},
+  {"ffi_list_all_size",                         (DL_FUNC) &ffi_list_all_size, 4},
+  {"ffi_list_check_all_size",                   (DL_FUNC) &ffi_list_check_all_size, 4},
+  {"ffi_list_all_recyclable",                   (DL_FUNC) &ffi_list_all_recyclable, 4},
+  {"ffi_list_check_all_recyclable",             (DL_FUNC) &ffi_list_check_all_recyclable, 4},
   {"ffi_vec_set_intersect",                     (DL_FUNC) &ffi_vec_set_intersect, 4},
   {"ffi_vec_set_difference",                    (DL_FUNC) &ffi_vec_set_difference, 4},
   {"ffi_vec_set_union",                         (DL_FUNC) &ffi_vec_set_union, 4},
   {"ffi_vec_set_symmetric_difference",          (DL_FUNC) &ffi_vec_set_symmetric_difference, 4},
   {"ffi_vec_expand_grid",                       (DL_FUNC) &ffi_vec_expand_grid, 4},
+  {"ffi_list_combine",                          (DL_FUNC) &ffi_list_combine, 11},
+  {"ffi_compact_seq",                           (DL_FUNC) &ffi_compact_seq, 3},
+  {"ffi_as_compact_condition",                  (DL_FUNC) &ffi_as_compact_condition, 1},
+  {"ffi_vec_case_when",                         (DL_FUNC) &ffi_vec_case_when, 7},
+  {"ffi_vec_replace_when",                      (DL_FUNC) &ffi_vec_replace_when, 4},
+  {"ffi_vec_recode_values",                     (DL_FUNC) &ffi_vec_recode_values, 9},
+  {"ffi_vec_replace_values",                    (DL_FUNC) &ffi_vec_replace_values, 6},
+  {"ffi_vec_if_else",                           (DL_FUNC) &ffi_vec_if_else, 6},
+  {"ffi_vec_pany",                              (DL_FUNC) &ffi_vec_pany, 4},
+  {"ffi_vec_pall",                              (DL_FUNC) &ffi_vec_pall, 4},
   {"ffi_exp_vec_cast",                          (DL_FUNC) &exp_vec_cast, 2},
   {NULL, NULL, 0}
 };
 
 extern r_obj* ffi_ptype_common(r_obj*, r_obj*, r_obj*, r_obj*);
-extern r_obj* ffi_ptype_common_opts(r_obj*, r_obj*, r_obj*, r_obj*);
+extern r_obj* ffi_ptype_common_params(r_obj*, r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_size_common(r_obj*, r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_recycle_common(r_obj*, r_obj*, r_obj*, r_obj*);
 extern r_obj* ffi_cast_common(r_obj*, r_obj*, r_obj*, r_obj*);
@@ -367,15 +400,15 @@ extern r_obj* ffi_new_data_frame(r_obj*);
 
 static
 const R_ExternalMethodDef ExtEntries[] = {
-  {"ffi_ptype_common",                 (DL_FUNC) &ffi_ptype_common, 1},
-  {"ffi_ptype_common_opts",            (DL_FUNC) &ffi_ptype_common_opts, 2},
-  {"ffi_size_common",                  (DL_FUNC) &ffi_size_common, 2},
-  {"ffi_recycle_common",               (DL_FUNC) &ffi_recycle_common, 1},
-  {"ffi_cast_common",                  (DL_FUNC) &ffi_cast_common, 1},
-  {"ffi_cast_common_opts",             (DL_FUNC) &ffi_cast_common_opts, 2},
-  {"ffi_rbind",                        (DL_FUNC) &ffi_rbind, 4},
-  {"ffi_cbind",                        (DL_FUNC) &ffi_cbind, 3},
-  {"ffi_vec_c",                        (DL_FUNC) &ffi_vec_c, 3},
+  {"ffi_ptype_common",                 (DL_FUNC) &ffi_ptype_common, 3},
+  {"ffi_ptype_common_params",          (DL_FUNC) &ffi_ptype_common_params, 4},
+  {"ffi_size_common",                  (DL_FUNC) &ffi_size_common, 3},
+  {"ffi_recycle_common",               (DL_FUNC) &ffi_recycle_common, 2},
+  {"ffi_cast_common",                  (DL_FUNC) &ffi_cast_common, 2},
+  {"ffi_cast_common_opts",             (DL_FUNC) &ffi_cast_common_opts, 3},
+  {"ffi_rbind",                        (DL_FUNC) &ffi_rbind, 5},
+  {"ffi_cbind",                        (DL_FUNC) &ffi_cbind, 4},
+  {"ffi_vec_c",                        (DL_FUNC) &ffi_vec_c, 4},
   {"ffi_new_data_frame",               (DL_FUNC) &ffi_new_data_frame, -1},
   {NULL, NULL, 0}
 };
@@ -387,14 +420,13 @@ export void R_init_vctrs(DllInfo *dll)
 
     // Maturing
     // In the public header
-    R_RegisterCCallable("vctrs", "obj_is_vector",      (DL_FUNC) &obj_is_vector);
-    R_RegisterCCallable("vctrs", "short_vec_size",     (DL_FUNC) &short_vec_size);
-    R_RegisterCCallable("vctrs", "short_vec_recycle",  (DL_FUNC) &short_vec_recycle);
+    R_RegisterCCallable("vctrs", "obj_is_vector",      (DL_FUNC) &maturing_obj_is_vector);
+    R_RegisterCCallable("vctrs", "short_vec_size",     (DL_FUNC) &maturing_short_vec_size);
+    R_RegisterCCallable("vctrs", "short_vec_recycle",  (DL_FUNC) &maturing_short_vec_recycle);
 
-    // Deprecated
-    // In the public header
-    // See `inst/include/vctrs.h` for details
-    R_RegisterCCallable("vctrs", "vec_is_vector", (DL_FUNC) &obj_is_vector);
+    // Defunct
+    // Previously were in the public header, but have since been removed
+    R_RegisterCCallable("vctrs", "vec_is_vector", (DL_FUNC) &defunct_vec_is_vector);
 
     // Experimental
     // Exported but not available in the public header
