@@ -1,8 +1,16 @@
+// IWYU pragma: private; include "rlang.h"
+
 #ifndef RLANG_ENV_H
 #define RLANG_ENV_H
 
-#include <stdbool.h>
-#include <Rversion.h>
+#include "rlang-types.h"
+#include "cnd.h"
+#include "globals.h"
+#include "obj.h"
+#include "rlang.h"
+#include "sym.h"
+
+#define RLANG_USE_R_EXISTS (R_VERSION < R_Version(4, 2, 0))
 
 
 extern r_obj* r_methods_ns_env;
@@ -26,11 +34,11 @@ r_obj* r_env_parent(r_obj* env) {
   if (env == r_envs.empty) {
     r_stop_internal("Can't take the parent of the empty environment.");
   }
+#if R_VERSION >= R_Version(4, 5, 0)
+  return R_ParentEnv(env);
+#else
   return ENCLOS(env);
-}
-static inline
-void r_env_poke_parent(r_obj* env, r_obj* new_parent) {
-  SET_ENCLOS(env, new_parent);
+#endif
 }
 
 static inline
@@ -50,11 +58,55 @@ static inline
 r_obj* r_env_find_anywhere(r_obj* env, r_obj* sym) {
   return Rf_findVar(sym, env);
 }
-r_obj* r_env_find_until(r_obj* env, r_obj* sym, r_obj* last);
 
+#if 1 || R_VERSION < R_Version(4, 5, 0)
+// We currently can't use `R_getVar()` which:
+// 1. Throws if not found
+// 2. Throws if argument is the missing arg
+// 3. Evaluates promises
+// Our operators have to return missing arguments.
+static inline
+r_obj* r_env_get(r_obj* env, r_obj* sym) {
+  r_obj* out = r_env_find(env, sym);
 
-// TODO: Enable `R_existsVarInFrame()` when R 4.2 is out
-#define RLANG_USE_R_EXISTS (1 || R_VERSION < R_Version(4, 2, 0))
+  if (out == r_syms.unbound) {
+    r_abort("object '%s' not found", r_sym_c_string(sym));
+  }
+
+  if (r_typeof(out) == R_TYPE_promise) {
+    KEEP(out);
+    out = Rf_eval(out, env);
+    FREE(1);
+    return out;
+  }
+
+  return out;
+}
+
+static inline
+r_obj* r_env_get_anywhere(r_obj* env, r_obj* sym) {
+  r_obj* out = r_env_find_anywhere(env, sym);
+
+  if (out == r_syms.unbound) {
+    r_abort("object '%s' not found", r_sym_c_string(sym));
+  }
+
+  return out;
+}
+#else
+static inline
+r_obj* r_env_get(r_obj* env, r_obj* sym) {
+  return R_getVar(sym, env, FALSE);
+}
+
+static inline
+r_obj* r_env_get_anywhere(r_obj* env, r_obj* sym) {
+  return R_getVar(sym, env, TRUE);
+}
+#endif
+
+r_obj* r_env_get_until(r_obj* env, r_obj* sym, r_obj* last);
+bool r_env_has_until(r_obj* env, r_obj* sym, r_obj* last);
 
 static inline
 bool r_env_has(r_obj* env, r_obj* sym) {
@@ -72,15 +124,38 @@ bool r_env_has_anywhere(r_obj* env, r_obj* sym) {
   bool r__env_has_anywhere(r_obj*, r_obj*);
   return r__env_has_anywhere(env, sym);
 #else
-  return TODO();
+  while (env != r_envs.empty) {
+    if (r_env_has(env, sym)) {
+      return true;
+    }
+    env = r_env_parent(env);
+  }
+  return false;
 #endif
 }
 
+bool r_env_has_missing(r_obj* env, r_obj* sym);
+
 r_obj* r_ns_env(const char* pkg);
 r_obj* r_base_ns_get(const char* name);
 
 r_obj* r_alloc_environment(r_ssize size, r_obj* parent);
 
+static inline
+r_obj* r_alloc_empty_environment(r_obj* parent) {
+  // Non-hashed environment.
+  // Very fast and useful when you aren't getting/setting from the result.
+#if R_VERSION >= R_Version(4, 1, 0)
+  const int hash = 0;
+  const int size = 0; // Not used when `hash = 0`
+  return R_NewEnv(parent, hash, size);
+#else
+  r_obj* env = Rf_allocSExp(R_TYPE_environment);
+  SET_ENCLOS(env, parent);
+  return env;
+#endif
+}
+
 r_obj* r_env_as_list(r_obj* x);
 r_obj* r_list_as_environment(r_obj* x, r_obj* parent);
 r_obj* r_env_clone(r_obj* env, r_obj* parent);
@@ -91,12 +166,7 @@ void r_env_coalesce(r_obj* env, r_obj* from);
 // Silently ignores bindings that are not defined in `env`.
 static inline
 void r_env_unbind(r_obj* env, r_obj* sym) {
-#if (R_VERSION < R_Version(4, 0, 0))
-  void r__env_unbind(r_obj*, r_obj*);
-  r__env_unbind(env, sym);
-#else
   R_removeVarFromFrame(sym, env);
-#endif
 }
 
 static inline
