@@ -1,4 +1,3 @@
-
 expect_lossy_cast <- function(expr) {
   cnd <- NULL
 
@@ -9,7 +8,7 @@ expect_lossy_cast <- function(expr) {
     }),
     expr
   )
-  expect_is(cnd, "vctrs_warning_cast_lossy")
+  expect_s3_class(cnd, "vctrs_warning_cast_lossy")
 
   out
 }
