@@ -72,13 +72,19 @@ test_that("base S3 casts report correct error call", {
 })
 
 test_that("names validation reports correct error call", {
-  my_function <- function() vec_as_names(c("x", "", "y"), repair = "check_unique")
+  my_function <- function() {
+    vec_as_names(c("x", "", "y"), repair = "check_unique")
+  }
   expect_snapshot((expect_error(my_function())))
 
-  my_function <- function() vec_as_names(c("x", "x"), repair = "check_unique", repair_arg = "repair")
+  my_function <- function() {
+    vec_as_names(c("x", "x"), repair = "check_unique", repair_arg = "repair")
+  }
   expect_snapshot((expect_error(my_function())))
 
-  my_function <- function() vec_as_names("...", repair = "check_unique", repair_arg = "repair")
+  my_function <- function() {
+    vec_as_names("...", repair = "check_unique", repair_arg = "repair")
+  }
   expect_snapshot((expect_error(my_function())))
 })
 
@@ -146,8 +152,12 @@ test_that("vec_size() reports error context", {
 test_that("vec_cast_common() reports error context", {
   my_function <- function(...) vec_cast_common(...)
   expect_snapshot((expect_error(my_function(my_arg = 1.5, .to = int()))))
-  expect_snapshot((expect_error(my_function(my_arg = 1.5, .to = int(), .arg = "my_arg"))))
-  expect_snapshot((expect_error(my_function(this_arg = 1, that_arg = "foo", .arg = "my_arg"))))
+  expect_snapshot(
+    (expect_error(my_function(my_arg = 1.5, .to = int(), .arg = "my_arg")))
+  )
+  expect_snapshot(
+    (expect_error(my_function(this_arg = 1, that_arg = "foo", .arg = "my_arg")))
+  )
   expect_snapshot((expect_error(my_function(1, "foo", .arg = "my_arg"))))
 
   x <- data.frame(x = "a")
@@ -158,6 +168,8 @@ test_that("vec_cast_common() reports error context", {
 test_that("vec_ptype_common() reports error context", {
   my_function <- function(...) vec_ptype_common(...)
   expect_snapshot((expect_error(my_function(this_arg = 1, that_arg = "foo"))))
-  expect_snapshot((expect_error(my_function(this_arg = 1, that_arg = "foo", .arg = "my_arg"))))
+  expect_snapshot(
+    (expect_error(my_function(this_arg = 1, that_arg = "foo", .arg = "my_arg")))
+  )
   expect_snapshot((expect_error(my_function(1, "foo", .arg = "my_arg"))))
 })
